@@ -416,7 +416,7 @@ impl<'a> Chars<'a> {
     }
 
     #[inline]
-    pub fn prev_impl(&mut self) -> Option<char> {
+    fn prev_impl(&mut self) -> Option<char> {
         // Put us back into a "prev" progression.
         if !self.last_call_was_prev_impl {
             self.chunk_iter.prev();
@@ -444,7 +444,7 @@ impl<'a> Chars<'a> {
     }
 
     #[inline]
-    pub fn next_impl(&mut self) -> Option<char> {
+    fn next_impl(&mut self) -> Option<char> {
         // Put us back into a "next" progression.
         if self.last_call_was_prev_impl {
             self.chunk_iter.next();
@@ -849,7 +849,7 @@ impl<'a> ExactSizeIterator for Lines<'a> {}
 /// `&str` slice for each one.  It is useful for situations such as:
 ///
 /// - Writing a rope's utf8 text data to disk (but see
-///   [`write_to()`](Rope::write_to) for a convenience function that does this
+///   [`write_to()`](crate::rope::Rope::write_to) for a convenience function that does this
 ///   for casual use-cases).
 /// - Streaming a rope's text data somewhere.
 /// - Saving a rope to a non-utf8 encoding, doing the encoding conversion
