@@ -24,7 +24,7 @@ use crate::{end_bound_to_num, start_bound_to_num, Error, Result};
 #[derive(Copy, Clone)]
 pub struct RopeSlice<'a>(pub(crate) RSEnum<'a>);
 
-#[derive(Copy, Clone)]
+#[derive(Copy, Clone, Debug)]
 pub(crate) enum RSEnum<'a> {
     Full {
         node: &'a Arc<Node>,
@@ -40,7 +40,13 @@ pub(crate) enum RSEnum<'a> {
 }
 
 impl<'a> RopeSlice<'a> {
-    pub(crate) fn new_with_range(node: &'a Arc<Node>, start: usize, end: usize) -> Self {
+    /// Used for tests and debugging purposes.
+    #[allow(dead_code)]
+    pub(crate) fn is_light(&self) -> bool {
+        matches!(&self.0, RSEnum::Light { .. })
+    }
+
+    pub(crate) fn new_with_range(node: &'a Arc<Node>, start: usize, end: usize) -> RopeSlice<'a> {
         assert!(start <= end);
         assert!(end <= node.text_info().chars as usize);
 
@@ -82,7 +88,7 @@ impl<'a> RopeSlice<'a> {
                 // Early out if we reach a leaf, because we can do the
                 // simpler lightweight slice then.
                 Node::Leaf(ref text) => {
-                    let start_byte = char_to_byte_idx(&text, n_start);
+                    let start_byte = char_to_byte_idx(text, n_start);
                     let end_byte =
                         start_byte + char_to_byte_idx(&text[start_byte..], n_end - n_start);
                     return RopeSlice(RSEnum::Light {
@@ -97,7 +103,7 @@ impl<'a> RopeSlice<'a> {
                 Node::Internal(ref children) => {
                     let mut start_char = 0;
                     for (i, inf) in children.info().iter().enumerate() {
-                        if n_start >= start_char && n_end < (start_char + inf.chars as usize) {
+                        if n_start >= start_char && n_end <= (start_char + inf.chars as usize) {
                             n_start -= start_char;
                             n_end -= start_char;
                             node = &children.nodes()[i];
@@ -118,6 +124,95 @@ impl<'a> RopeSlice<'a> {
         })
     }
 
+    pub(crate) fn new_with_byte_range(
+        node: &'a Arc<Node>,
+        start: usize,
+        end: usize,
+    ) -> Result<RopeSlice<'a>> {
+        assert!(start <= end);
+        assert!(end <= node.text_info().bytes as usize);
+
+        // Early-out shortcut for taking a slice of the full thing.
+        if start == 0 && end == node.byte_count() {
+            if node.is_leaf() {
+                let text = node.leaf_text();
+                return Ok(RopeSlice(RSEnum::Light {
+                    text,
+                    char_count: count_chars(text) as Count,
+                    utf16_surrogate_count: count_utf16_surrogates(text) as Count,
+                    line_break_count: count_line_breaks(text) as Count,
+                }));
+            } else {
+                return Ok(RopeSlice(RSEnum::Full {
+                    node,
+                    start_info: TextInfo {
+                        bytes: 0,
+                        chars: 0,
+                        utf16_surrogates: 0,
+                        line_breaks: 0,
+                    },
+                    end_info: TextInfo {
+                        bytes: node.byte_count() as Count,
+                        chars: node.char_count() as Count,
+                        utf16_surrogates: node.utf16_surrogate_count() as Count,
+                        line_breaks: node.line_break_count() as Count,
+                    },
+                }));
+            }
+        }
+
+        // Find the deepest node that still contains the full range given.
+        let mut n_start = start;
+        let mut n_end = end;
+        let mut node = node;
+        'outer: loop {
+            match *(node as &Node) {
+                // Early out if we reach a leaf, because we can do the
+                // simpler lightweight slice then.
+                Node::Leaf(ref text) => {
+                    let start_byte = n_start;
+                    let end_byte = n_end;
+                    if !text.is_char_boundary(start_byte) || !text.is_char_boundary(end_byte) {
+                        return Err(Error::ByteRangeNotCharBoundary(Some(start), Some(end)));
+                    }
+                    return Ok(RopeSlice(RSEnum::Light {
+                        text: &text[start_byte..end_byte],
+                        char_count: count_chars(&text[start_byte..end_byte]) as Count,
+                        utf16_surrogate_count: count_utf16_surrogates(&text[start_byte..end_byte])
+                            as Count,
+                        line_break_count: count_line_breaks(&text[start_byte..end_byte]) as Count,
+                    }));
+                }
+
+                Node::Internal(ref children) => {
+                    let mut start_byte = 0;
+                    for (i, inf) in children.info().iter().enumerate() {
+                        if n_start >= start_byte && n_end <= (start_byte + inf.bytes as usize) {
+                            n_start -= start_byte;
+                            n_end -= start_byte;
+                            node = &children.nodes()[i];
+                            continue 'outer;
+                        }
+                        start_byte += inf.bytes as usize;
+                    }
+                    break;
+                }
+            }
+        }
+
+        // Make sure the bytes indices are valid char boundaries.
+        if !node.is_char_boundary(n_start) || !node.is_char_boundary(n_end) {
+            return Err(Error::ByteRangeNotCharBoundary(Some(start), Some(end)));
+        }
+
+        // Create the slice
+        Ok(RopeSlice(RSEnum::Full {
+            node,
+            start_info: node.byte_to_text_info(n_start),
+            end_info: node.byte_to_text_info(n_end),
+        }))
+    }
+
     //-----------------------------------------------------------------------
     // Informational methods
 
@@ -526,7 +621,7 @@ impl<'a> RopeSlice<'a> {
     ///
     /// Panics if the start of the range is greater than the end, or the end
     /// is out of bounds (i.e. `end > len_chars()`).
-    pub fn slice<R>(&self, char_range: R) -> Self
+    pub fn slice<R>(&self, char_range: R) -> RopeSlice<'a>
     where
         R: RangeBounds<usize>,
     {
@@ -576,6 +671,28 @@ impl<'a> RopeSlice<'a> {
         }
     }
 
+    /// Returns a sub-slice of the `RopeSlice` in the given byte index range.
+    ///
+    /// Uses range syntax, e.g. `2..7`, `2..`, etc.
+    ///
+    /// Runs in O(log N) time.
+    ///
+    /// # Panics
+    ///
+    /// Panics if:
+    /// - The start of the range is greater than the end.
+    /// - The end is out of bounds (i.e. `end > len_bytes()`).
+    /// - The range doesn't align with char boundaries.
+    pub fn byte_slice<R>(&self, byte_range: R) -> RopeSlice<'a>
+    where
+        R: RangeBounds<usize>,
+    {
+        match self.get_byte_slice_impl(byte_range) {
+            Ok(s) => return s,
+            Err(e) => panic!("byte_slice(): {}", e),
+        }
+    }
+
     //-----------------------------------------------------------------------
     // Iterator methods
 
@@ -614,7 +731,7 @@ impl<'a> RopeSlice<'a> {
     ///
     /// Panics if `byte_idx` is out of bounds (i.e. `byte_idx > len_bytes()`).
     #[inline]
-    pub fn bytes_at(&self, byte_idx: usize) -> Bytes {
+    pub fn bytes_at(&self, byte_idx: usize) -> Bytes<'a> {
         if let Some(out) = self.get_bytes_at(byte_idx) {
             out
         } else {
@@ -661,7 +778,7 @@ impl<'a> RopeSlice<'a> {
     ///
     /// Panics if `char_idx` is out of bounds (i.e. `char_idx > len_chars()`).
     #[inline]
-    pub fn chars_at(&self, char_idx: usize) -> Chars {
+    pub fn chars_at(&self, char_idx: usize) -> Chars<'a> {
         if let Some(out) = self.get_chars_at(char_idx) {
             out
         } else {
@@ -707,7 +824,7 @@ impl<'a> RopeSlice<'a> {
     ///
     /// Panics if `line_idx` is out of bounds (i.e. `line_idx > len_lines()`).
     #[inline]
-    pub fn lines_at(&self, line_idx: usize) -> Lines {
+    pub fn lines_at(&self, line_idx: usize) -> Lines<'a> {
         if let Some(out) = self.get_lines_at(line_idx) {
             out
         } else {
@@ -824,7 +941,7 @@ impl<'a> RopeSlice<'a> {
     ///
     /// Panics if `line_break_idx` is out of bounds (i.e. `line_break_idx > len_lines()`).
     #[inline]
-    pub fn chunks_at_line_break(&self, line_break_idx: usize) -> (Chunks, usize, usize, usize) {
+    pub fn chunks_at_line_break(&self, line_break_idx: usize) -> (Chunks<'a>, usize, usize, usize) {
         if let Some(out) = self.get_chunks_at_line_break(line_break_idx) {
             out
         } else {
@@ -898,9 +1015,7 @@ impl<'a> RopeSlice<'a> {
         if char_idx <= self.len_chars() {
             match *self {
                 RopeSlice(RSEnum::Full {
-                    ref node,
-                    start_info,
-                    ..
+                    node, start_info, ..
                 }) => {
                     let char_idx = char_idx + start_info.chars as usize;
 
@@ -934,9 +1049,7 @@ impl<'a> RopeSlice<'a> {
         if utf16_cu_idx <= self.len_utf16_cu() {
             match *self {
                 RopeSlice(RSEnum::Full {
-                    ref node,
-                    start_info,
-                    ..
+                    node, start_info, ..
                 }) => {
                     let utf16_cu_idx =
                         utf16_cu_idx + (start_info.chars + start_info.utf16_surrogates) as usize;
@@ -1174,7 +1287,7 @@ impl<'a> RopeSlice<'a> {
     }
 
     /// Non-panicking version of [`slice()`](RopeSlice::slice).
-    pub fn get_slice<R>(&self, char_range: R) -> Option<Self>
+    pub fn get_slice<R>(&self, char_range: R) -> Option<RopeSlice<'a>>
     where
         R: RangeBounds<usize>,
     {
@@ -1220,9 +1333,96 @@ impl<'a> RopeSlice<'a> {
         }
     }
 
+    /// Non-panicking version of [`byte_slice()`](RopeSlice::byte_slice).
+    pub fn get_byte_slice<R>(&self, byte_range: R) -> Option<RopeSlice<'a>>
+    where
+        R: RangeBounds<usize>,
+    {
+        self.get_byte_slice_impl(byte_range).ok()
+    }
+
+    pub(crate) fn get_byte_slice_impl<R>(&self, byte_range: R) -> Result<RopeSlice<'a>>
+    where
+        R: RangeBounds<usize>,
+    {
+        let start_range = start_bound_to_num(byte_range.start_bound());
+        let end_range = end_bound_to_num(byte_range.end_bound());
+
+        // Bounds checks.
+        match (start_range, end_range) {
+            (None, None) => {
+                // Early-out shortcut for taking a slice of the full thing.
+                return Ok(*self);
+            }
+            (Some(s), Some(e)) => {
+                if s > e {
+                    return Err(Error::ByteRangeInvalid(s, e));
+                } else if e > self.len_bytes() {
+                    return Err(Error::ByteRangeOutOfBounds(
+                        start_range,
+                        end_range,
+                        self.len_bytes(),
+                    ));
+                }
+            }
+            (Some(s), None) => {
+                if s > self.len_bytes() {
+                    return Err(Error::ByteRangeOutOfBounds(
+                        start_range,
+                        end_range,
+                        self.len_bytes(),
+                    ));
+                }
+            }
+            (None, Some(e)) => {
+                if e > self.len_bytes() {
+                    return Err(Error::ByteRangeOutOfBounds(
+                        start_range,
+                        end_range,
+                        self.len_bytes(),
+                    ));
+                }
+            }
+        }
+
+        let (start, end) = (
+            start_range.unwrap_or(0),
+            end_range.unwrap_or_else(|| self.len_bytes()),
+        );
+
+        match *self {
+            RopeSlice(RSEnum::Full {
+                node, start_info, ..
+            }) => RopeSlice::new_with_byte_range(
+                node,
+                start_info.bytes as usize + start,
+                start_info.bytes as usize + end,
+            )
+            .map_err(|e| {
+                if let Error::ByteRangeNotCharBoundary(_, _) = e {
+                    Error::ByteRangeNotCharBoundary(start_range, end_range)
+                } else {
+                    e
+                }
+            }),
+            RopeSlice(RSEnum::Light { text, .. }) => {
+                if !text.is_char_boundary(start) || !text.is_char_boundary(end) {
+                    return Err(Error::ByteRangeNotCharBoundary(start_range, end_range));
+                }
+                let new_text = &text[start..end];
+                Ok(RopeSlice(RSEnum::Light {
+                    text: new_text,
+                    char_count: count_chars(new_text) as Count,
+                    utf16_surrogate_count: count_utf16_surrogates(new_text) as Count,
+                    line_break_count: count_line_breaks(new_text) as Count,
+                }))
+            }
+        }
+    }
+
     /// Non-panicking version of [`bytes_at()`](RopeSlice::bytes_at).
     #[inline]
-    pub fn get_bytes_at(&self, byte_idx: usize) -> Option<Bytes> {
+    pub fn get_bytes_at(&self, byte_idx: usize) -> Option<Bytes<'a>> {
         // Bounds check
         if byte_idx <= self.len_bytes() {
             match *self {
@@ -1249,7 +1449,7 @@ impl<'a> RopeSlice<'a> {
 
     /// Non-panicking version of [`chars_at()`](RopeSlice::chars_at).
     #[inline]
-    pub fn get_chars_at(&self, char_idx: usize) -> Option<Chars> {
+    pub fn get_chars_at(&self, char_idx: usize) -> Option<Chars<'a>> {
         // Bounds check
         if char_idx <= self.len_chars() {
             match *self {
@@ -1276,7 +1476,7 @@ impl<'a> RopeSlice<'a> {
 
     /// Non-panicking version of [`lines_at()`](RopeSlice::lines_at).
     #[inline]
-    pub fn get_lines_at(&self, line_idx: usize) -> Option<Lines> {
+    pub fn get_lines_at(&self, line_idx: usize) -> Option<Lines<'a>> {
         // Bounds check
         if line_idx <= self.len_lines() {
             match *self {
@@ -1415,7 +1615,7 @@ impl<'a> RopeSlice<'a> {
     pub fn get_chunks_at_line_break(
         &self,
         line_break_idx: usize,
-    ) -> Option<(Chunks, usize, usize, usize)> {
+    ) -> Option<(Chunks<'a>, usize, usize, usize)> {
         // Bounds check
         if line_break_idx <= self.len_lines() {
             match *self {
@@ -1759,6 +1959,19 @@ impl<'a, 'b> std::cmp::PartialOrd<RopeSlice<'b>> for RopeSlice<'a> {
     }
 }
 
+impl<'a> std::hash::Hash for RopeSlice<'a> {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        for chunk in self.chunks() {
+            state.write(chunk.as_bytes());
+        }
+
+        // Same strategy as `&str` in stdlib, so that e.g. two adjacent
+        // fields in a `#[derive(Hash)]` struct with "Hi " and "there"
+        // vs "Hi t" and "here" give the struct a different hash.
+        state.write_u8(0xff)
+    }
+}
+
 //===========================================================
 
 #[cfg(test)]
@@ -1767,6 +1980,7 @@ mod tests {
         byte_to_char_idx, byte_to_line_idx, char_to_byte_idx, char_to_line_idx,
     };
     use crate::Rope;
+    use std::hash::{Hash, Hasher};
 
     // 127 bytes, 103 chars, 1 line
     const TEXT: &str = "Hello there!  How're you doing?  It's \
@@ -2525,16 +2739,18 @@ mod tests {
 
         let s2 = s1.slice(21..21);
 
+        assert!(s2.is_light());
         assert_eq!("", s2);
     }
 
     #[test]
-    #[should_panic]
     fn slice_05() {
         let r = Rope::from_str(TEXT);
-        let s = r.slice(5..43);
-
-        s.slice(21..20);
+        let s1 = r.slice(5..98);
+        for i in 0..(s1.len_chars() - 1) {
+            let s2 = s1.slice(i..(i + 1));
+            assert!(s2.is_light());
+        }
     }
 
     #[test]
@@ -2543,9 +2759,107 @@ mod tests {
         let r = Rope::from_str(TEXT);
         let s = r.slice(5..43);
 
+        s.slice(21..20);
+    }
+
+    #[test]
+    #[should_panic]
+    fn slice_07() {
+        let r = Rope::from_str(TEXT);
+        let s = r.slice(5..43);
+
         s.slice(37..39);
     }
 
+    #[test]
+    fn byte_slice_01() {
+        let r = Rope::from_str(TEXT);
+        let s1 = r.byte_slice(..);
+
+        let s2 = s1.byte_slice(..);
+
+        assert_eq!(TEXT, s2);
+    }
+
+    #[test]
+    fn byte_slice_02() {
+        let r = Rope::from_str(TEXT);
+        let s1 = r.byte_slice(50..118);
+
+        let s2 = s1.byte_slice(3..25);
+
+        assert_eq!(&TEXT[53..75], s2);
+    }
+
+    #[test]
+    fn byte_slice_03() {
+        let r = Rope::from_str(TEXT);
+        let s1 = r.byte_slice(50..118);
+
+        let s2 = s1.byte_slice(7..65);
+
+        assert_eq!(&TEXT[57..115], s2);
+    }
+
+    #[test]
+    fn byte_slice_04() {
+        let r = Rope::from_str(TEXT);
+        let s1 = r.byte_slice(50..118);
+
+        let s2 = s1.byte_slice(21..21);
+
+        assert!(s2.is_light());
+        assert_eq!("", s2);
+    }
+
+    #[test]
+    fn byte_slice_05() {
+        let r = Rope::from_str(TEXT);
+        let s1 = r.byte_slice(4..86);
+        for i in 0..(s1.len_bytes() - 1) {
+            let s2 = s1.byte_slice(i..(i + 1));
+            assert!(s2.is_light());
+        }
+    }
+
+    #[test]
+    #[should_panic]
+    fn byte_slice_06() {
+        let r = Rope::from_str(TEXT);
+        let s = r.byte_slice(50..118);
+
+        s.byte_slice(21..20);
+    }
+
+    #[test]
+    #[should_panic]
+    fn byte_slice_07() {
+        let r = Rope::from_str(TEXT);
+        let s = r.byte_slice(50..85);
+
+        s.byte_slice(35..36);
+    }
+
+    #[test]
+    #[should_panic]
+    fn byte_slice_08() {
+        let r = Rope::from_str(TEXT);
+        let s = r.byte_slice(50..118);
+
+        // Not a char boundary.
+        s.byte_slice(..43);
+    }
+
+    #[test]
+    #[should_panic]
+    fn byte_slice_09() {
+        let r = Rope::from_str(TEXT);
+        let s = r.byte_slice(50..118);
+
+        // Not a char boundary.
+        s.byte_slice(43..);
+    }
+
     #[test]
     fn eq_str_01() {
         let r = Rope::from_str(TEXT);
@@ -2713,5 +3027,18 @@ mod tests {
         assert_eq!(s, cow);
     }
 
+    #[test]
+    fn hash_01() {
+        let mut h1 = std::collections::hash_map::DefaultHasher::new();
+        let mut h2 = std::collections::hash_map::DefaultHasher::new();
+        let r = Rope::from_str("Hello there!");
+        let s = r.slice(..);
+
+        r.hash(&mut h1);
+        s.hash(&mut h2);
+
+        assert_eq!(h1.finish(), h2.finish());
+    }
+
     // Iterator tests are in the iter module
 }
