@@ -1,6 +1,8 @@
 use std::sync::Arc;
 
-use crate::str_utils::{byte_to_line_idx, byte_to_utf16_surrogate_idx, char_to_byte_idx};
+use crate::str_utils::{
+    byte_to_char_idx, byte_to_line_idx, byte_to_utf16_surrogate_idx, char_to_byte_idx,
+};
 use crate::tree::node_text::fix_segment_seam;
 use crate::tree::{
     Count, NodeChildren, NodeText, TextInfo, MAX_BYTES, MAX_CHILDREN, MIN_BYTES, MIN_CHILDREN,
@@ -127,7 +129,7 @@ impl Node {
     /// Returns (in this order):
     /// - The updated TextInfo for the node.
     /// - Whether there's a possible CRLF seam that needs fixing.
-    /// - Whether fix_after_remove() needs to be run after this.
+    /// - Whether fix_tree_seam() needs to be run after this.
     ///
     /// WARNING: does not correctly handle all text being removed.  That
     /// should be special-cased in calling code.
@@ -156,25 +158,29 @@ impl Node {
 
                     let seg_len = byte_end - byte_start; // Length of removal segement
                     if seg_len < (leaf_text.len() - seg_len) {
+                        #[allow(unused_mut)]
                         let mut info =
                             node_info - TextInfo::from_str(&leaf_text[byte_start..byte_end]);
 
                         // Check for CRLF pairs on the removal seams, and
                         // adjust line break counts accordingly.
-                        if byte_end < leaf_text.len()
-                            && leaf_text.as_bytes()[byte_end - 1] == 0x0D
-                            && leaf_text.as_bytes()[byte_end] == 0x0A
+                        #[cfg(any(feature = "cr_lines", feature = "unicode_lines"))]
                         {
-                            info.line_breaks += 1;
-                        }
-                        if byte_start > 0 && leaf_text.as_bytes()[byte_start - 1] == 0x0D {
-                            if leaf_text.as_bytes()[byte_start] == 0x0A {
+                            if byte_end < leaf_text.len()
+                                && leaf_text.as_bytes()[byte_end - 1] == 0x0D
+                                && leaf_text.as_bytes()[byte_end] == 0x0A
+                            {
                                 info.line_breaks += 1;
                             }
-
-                            if byte_end < leaf_text.len() && leaf_text.as_bytes()[byte_end] == 0x0A
-                            {
-                                info.line_breaks -= 1;
+                            if byte_start > 0 && leaf_text.as_bytes()[byte_start - 1] == 0x0D {
+                                if leaf_text.as_bytes()[byte_start] == 0x0A {
+                                    info.line_breaks += 1;
+                                }
+                                if byte_end < leaf_text.len()
+                                    && leaf_text.as_bytes()[byte_end] == 0x0A
+                                {
+                                    info.line_breaks -= 1;
+                                }
                             }
                         }
 
@@ -186,7 +192,7 @@ impl Node {
                         // Remove the text
                         leaf_text.remove_range(byte_start, byte_end);
 
-                        (TextInfo::from_str(&leaf_text), seam, false)
+                        (TextInfo::from_str(leaf_text), seam, false)
                     }
                 } else {
                     // Remove all of the text
@@ -555,6 +561,21 @@ impl Node {
         }
     }
 
+    /// Returns the TextInfo at the given byte index.
+    #[inline(always)]
+    pub fn byte_to_text_info(&self, byte_idx: usize) -> TextInfo {
+        let (chunk, info) = self.get_chunk_at_byte(byte_idx);
+        let bi = byte_idx - info.bytes as usize;
+        let ci = byte_to_char_idx(chunk, byte_idx - info.bytes as usize);
+        TextInfo {
+            bytes: byte_idx as Count,
+            chars: info.chars + ci as Count,
+            utf16_surrogates: info.utf16_surrogates
+                + byte_to_utf16_surrogate_idx(chunk, bi) as Count,
+            line_breaks: info.line_breaks + byte_to_line_idx(chunk, bi) as Count,
+        }
+    }
+
     pub fn text_info(&self) -> TextInfo {
         match *self {
             Node::Leaf(ref text) => TextInfo::from_str(text),
@@ -562,6 +583,11 @@ impl Node {
         }
     }
 
+    pub fn is_char_boundary(&self, byte_idx: usize) -> bool {
+        let (chunk, info) = self.get_chunk_at_byte(byte_idx);
+        chunk.is_char_boundary(byte_idx - info.bytes as usize)
+    }
+
     //-----------------------------------------
 
     pub fn child_count(&self) -> usize {
@@ -889,13 +915,13 @@ impl Node {
         }
     }
 
-    /// Fixes up the tree after remove_char_range().
+    /// Fixes up the tree after remove_char_range() or Rope::append().
     ///
     /// Takes the char index of the start of the removal range.
     ///
     /// Returns whether it did anything or not that would affect the
     /// parent. True: did stuff, false: didn't do stuff
-    pub fn fix_after_remove(&mut self, char_idx: usize) -> bool {
+    pub fn fix_tree_seam(&mut self, char_idx: usize) -> bool {
         if let Node::Internal(ref mut children) = *self {
             let mut did_stuff = false;
             loop {
@@ -939,14 +965,14 @@ impl Node {
                 if end_info.chars as usize == char_idx && (child_i + 1) < children.len() {
                     let tmp = children.info()[child_i].chars as usize;
                     let effect_1 =
-                        Arc::make_mut(&mut children.nodes_mut()[child_i]).fix_after_remove(tmp);
+                        Arc::make_mut(&mut children.nodes_mut()[child_i]).fix_tree_seam(tmp);
                     let effect_2 =
-                        Arc::make_mut(&mut children.nodes_mut()[child_i + 1]).fix_after_remove(0);
+                        Arc::make_mut(&mut children.nodes_mut()[child_i + 1]).fix_tree_seam(0);
                     if (!effect_1) && (!effect_2) {
                         break;
                     }
                 } else if !Arc::make_mut(&mut children.nodes_mut()[child_i])
-                    .fix_after_remove(char_idx - start_info.chars as usize)
+                    .fix_tree_seam(char_idx - start_info.chars as usize)
                 {
                     break;
                 }
