@@ -103,7 +103,7 @@ impl NodeChildren {
                             text1.push_str(text2);
                             true
                         } else {
-                            let right = text1.push_str_split(&text2);
+                            let right = text1.push_str_split(text2);
                             *text2 = right;
                             false
                         }
@@ -559,31 +559,31 @@ mod inner {
 
         /// Access to the nodes array.
         #[inline(always)]
-        pub fn nodes<'a>(&'a self) -> &'a [Arc<Node>] {
+        pub fn nodes(&self) -> &[Arc<Node>] {
             unsafe { mem::transmute(&self.nodes[..(self.len())]) }
         }
 
         /// Mutable access to the nodes array.
         #[inline(always)]
-        pub fn nodes_mut<'a>(&'a mut self) -> &'a mut [Arc<Node>] {
+        pub fn nodes_mut(&mut self) -> &mut [Arc<Node>] {
             unsafe { mem::transmute(&mut self.nodes[..(self.len as usize)]) }
         }
 
         /// Access to the info array.
         #[inline(always)]
-        pub fn info<'a>(&'a self) -> &'a [TextInfo] {
+        pub fn info(&self) -> &[TextInfo] {
             unsafe { mem::transmute(&self.info[..(self.len())]) }
         }
 
         /// Mutable access to the info array.
         #[inline(always)]
-        pub fn info_mut<'a>(&'a mut self) -> &'a mut [TextInfo] {
+        pub fn info_mut(&mut self) -> &mut [TextInfo] {
             unsafe { mem::transmute(&mut self.info[..(self.len as usize)]) }
         }
 
         /// Mutable access to both the info and nodes arrays simultaneously.
         #[inline(always)]
-        pub fn data_mut<'a>(&'a mut self) -> (&'a mut [TextInfo], &'a mut [Arc<Node>]) {
+        pub fn data_mut(&mut self) -> (&mut [TextInfo], &mut [Arc<Node>]) {
             (
                 unsafe { mem::transmute(&mut self.info[..(self.len as usize)]) },
                 unsafe { mem::transmute(&mut self.nodes[..(self.len as usize)]) },
@@ -625,22 +625,17 @@ mod inner {
             let len = self.len();
             // This unsafe code simply shifts the elements of the arrays over
             // to make space for the new inserted value.  The `.info` array
-            // shifting can be replaced with a safe call to `copy_within()`
-            // once that API is stabalized in the standard library.  However,
-            // the `.nodes` array shift cannot, because of the specific drop
-            // semantics needed for safety.
+            // shifting can be done with a safe call to `copy_within()`.
+            // However, the `.nodes` array shift cannot, because of the
+            // specific drop semantics needed for safety.
             unsafe {
                 ptr::copy(
                     self.nodes.as_ptr().add(idx),
                     self.nodes.as_mut_ptr().add(idx + 1),
                     len - idx,
                 );
-                ptr::copy(
-                    self.info.as_ptr().add(idx),
-                    self.info.as_mut_ptr().add(idx + 1),
-                    len - idx,
-                );
             }
+            self.info.copy_within(idx..len, idx + 1);
 
             self.info[idx] = MaybeUninit::new(item.0);
             self.nodes[idx] = MaybeUninit::new(item.1);
@@ -663,22 +658,17 @@ mod inner {
             let len = self.len();
             // This unsafe code simply shifts the elements of the arrays over
             // to fill in the gap left by the removed element.  The `.info`
-            // array shifting can be replaced with a safe call to
-            // `copy_within()` once that API is stabalized in the standard
-            // library.  However, the `.nodes` array shift cannot, because of
-            // the specific drop semantics needed for safety.
+            // array shifting can be done with a safe call to `copy_within()`.
+            // However, the `.nodes` array shift cannot, because of the
+            // specific drop semantics needed for safety.
             unsafe {
                 ptr::copy(
                     self.nodes.as_ptr().add(idx + 1),
                     self.nodes.as_mut_ptr().add(idx),
                     len - idx - 1,
                 );
-                ptr::copy(
-                    self.info.as_ptr().add(idx + 1),
-                    self.info.as_mut_ptr().add(idx),
-                    len - idx - 1,
-                );
             }
+            self.info.copy_within((idx + 1)..len, idx);
 
             self.len -= 1;
             return item;
